@@ -5,7 +5,7 @@
 
 /* nettle, low-level cryptographics library
  *
- * Copyright (C) 2002 Niels Mller
+ * Copyright (C) 2002 Niels Möller
  *  
  * The nettle library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -31,17 +31,15 @@
 
 #include "aes-internal.h"
 
-/* On my sparc, encryption is significantly slower than decryption,
- * even though the *only* difference is which table is passed to _aes_crypt.
- *
- * Really strange.
- */
+/* The main point on this function is to help the assembler
+   implementations of _nettle_aes_encrypt to get the table pointer.
+   For PIC code, the details can be complex and system dependent. */
 void
-aes_encrypt(struct aes_ctx *ctx,
+aes_encrypt(const struct aes_ctx *ctx,
 	    unsigned length, uint8_t *dst,
 	    const uint8_t *src)
 {
   assert(!(length % AES_BLOCK_SIZE) );
-  _aes_crypt(ctx, &_aes_encrypt_table,
-	     length, dst, src);
+  _aes_encrypt(ctx, &_aes_encrypt_table,
+	       length, dst, src);
 }
