@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: acconfig.h,v 1.161 2009/03/17 19:07:44 grubba Exp $
+|| $Id: 5a0daeea51367e24219b6164d4daafad1d37b5eb $
 */
 
 #ifndef MACHINE_H
@@ -636,4 +636,41 @@
  */
 #define PIKE_OOB_WORKS -1
 
+/* dlmalloc has mallinfo. */
+#if defined(USE_DL_MALLOC) && !defined(HAVE_MALLINFO)
+#define HAVE_MALLINFO
+
+#if defined (HAVE_MALLOC_H) && defined (HAVE_STRUCT_MALLINFO)
+#include <malloc.h>
+#else /* HAVE_MALLOC_H && HAVE_STRUCT_MALLINFO */
+
+#ifndef MALLINFO_FIELD_TYPE
+#define MALLINFO_FIELD_TYPE size_t
+#endif  /* MALLINFO_FIELD_TYPE */
+
+#ifdef HAVE_STDDEF_H
+/* Needed for size_t. */
+#include <stddef.h>
+#endif
+
+/* dlmalloc definition of struct mallinfo. */
+struct mallinfo {
+  MALLINFO_FIELD_TYPE arena;    /* non-mmapped space allocated from system */
+  MALLINFO_FIELD_TYPE ordblks;  /* number of free chunks */
+  MALLINFO_FIELD_TYPE smblks;   /* always 0 */
+  MALLINFO_FIELD_TYPE hblks;    /* always 0 */
+  MALLINFO_FIELD_TYPE hblkhd;   /* space in mmapped regions */
+  MALLINFO_FIELD_TYPE usmblks;  /* maximum total allocated space */
+  MALLINFO_FIELD_TYPE fsmblks;  /* always 0 */
+  MALLINFO_FIELD_TYPE uordblks; /* total allocated space */
+  MALLINFO_FIELD_TYPE fordblks; /* total free space */
+  MALLINFO_FIELD_TYPE keepcost; /* releasable (via malloc_trim) space */
+};
+
+struct mallinfo dlmallinfo(void);
+
+#endif /* HAVE_USR_INCLUDE_MALLOC_H */
+
+#endif
+
 #endif /* MACHINE_H */
