@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: builtin_functions.h,v 1.37 2008/07/09 13:29:01 grubba Exp $
+|| $Id: d879bf216775cacbecf1f8a8d35ace3e58d7e592 $
 */
 
 #ifndef BUILTIN_EFUNS_H
@@ -46,6 +46,11 @@ struct replace_many_context
   int flags;
 };
 
+PMOD_EXPORT struct object *get_val_true(void);
+PMOD_EXPORT struct object *get_val_false(void);
+PMOD_EXPORT struct object *get_val_null(void);
+PMOD_EXPORT struct program *get_sql_null_prog(void);
+
 PMOD_EXPORT void f_equal(INT32 args);
 PMOD_EXPORT void f_hash(INT32 args);
 PMOD_EXPORT void f_copy_value(INT32 args);
@@ -159,6 +164,7 @@ PMOD_EXPORT void f_filter(INT32 args);
 PMOD_EXPORT void f_inherit_list(INT32 args);
 PMOD_EXPORT void f_function_defined(INT32 args);
 void init_builtin_efuns(void);
+void exit_builtin_efuns(void);
 
 /* From iterators.cmod. */
 PMOD_EXPORT void f_get_iterator(INT32 args);
@@ -179,6 +185,7 @@ PMOD_EXPORT void f_ctime(INT32 args);
 PMOD_EXPORT void f_mkmapping(INT32 args);
 PMOD_EXPORT void f_string_count(INT32 args);
 PMOD_EXPORT void f_string_trim_whites(INT32 args);
+PMOD_EXPORT void f_string_normalize_space(INT32 args);
 PMOD_EXPORT void f_string_trim_all_whites(INT32 args);
 PMOD_EXPORT void f_program_implements(INT32 args);
 PMOD_EXPORT void f_program_inherits(INT32 args);
