@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: docode.c,v 1.204 2008/08/28 15:13:19 grubba Exp $
+|| $Id: 786c60aaf38bf4856f007e8b84bd6bd8c8bcacf7 $
 */
 
 #include "global.h"
@@ -870,6 +870,7 @@ static int do_docode2(node *n, int flags)
       int level = 0;
       struct program_state *state = Pike_compiler;
       while (state && (state->new_program->id != n->u.integer.a)) {
+	state->new_program->flags |= PROGRAM_USES_PARENT | PROGRAM_NEEDS_PARENT;
 	state = state->previous;
 	level++;
       }
@@ -920,7 +921,27 @@ static int do_docode2(node *n, int flags)
 	    emit2(F_EXTERNAL, n->u.integer.b, level);
 	  }
 	} else if (level) {
-	  emit2(F_EXTERNAL, n->u.integer.b, level);
+	  if (IDENTIFIER_IS_CONSTANT(id->identifier_flags) &&
+	      (ref->id_flags & ID_INLINE) && !ref->inherit_offset &&
+	      (id->func.offset >= 0)) {
+	    /* An inline, local or final constant identifier in
+	     * a lexically surrounding (aka parent) class.
+	     * Avoid vtable traversal during runtime by moving
+	     * the constant to this class.
+	     */
+	    struct svalue *s = &state->new_program->
+	      constants[id->func.offset].sval;
+	    if (s->type == T_PROGRAM &&
+		s->u.program->flags & PROGRAM_USES_PARENT) {
+	      /* An external reference is required. */
+	      emit2(F_EXTERNAL, n->u.integer.b, level);
+	    } else {
+	      int tmp1 = store_constant(s, 1, NULL);
+	      emit1(F_CONSTANT, tmp1);
+	    }
+	  } else {
+	    emit2(F_EXTERNAL, n->u.integer.b, level);
+	  }
 	} else if (n->u.integer.b == IDREF_MAGIC_THIS) {
 	  emit1(F_THIS_OBJECT, 0);
 	} else if(IDENTIFIER_IS_FUNCTION(id->identifier_flags) &&
@@ -930,6 +951,21 @@ static int do_docode2(node *n, int flags)
 	   * can't zero, i.e. when we know the function isn't just a
 	   * prototype. */
 	  emit1(F_LFUN, n->u.integer.b);
+	} else if (IDENTIFIER_IS_CONSTANT(id->identifier_flags) &&
+		   (ref->id_flags & ID_INLINE) && !ref->inherit_offset &&
+		   (id->func.offset >= 0)) {
+	  /* An inline, local or final constant identifier.
+	   * No need for vtable traversal during runtime.
+	   */
+	  struct svalue *s = &state->new_program->
+	    constants[id->func.offset].sval;
+	  if (s->type == T_PROGRAM &&
+	      s->u.program->flags & PROGRAM_USES_PARENT) {
+	    /* Program using parent. Convert to an LFUN. */
+	    emit1(F_LFUN, n->u.integer.b);
+	  } else {
+	    emit1(F_CONSTANT, id->func.offset);
+	  }
 	}else{
 	  emit1(F_GLOBAL, n->u.integer.b);
 	}
@@ -997,7 +1033,6 @@ static int do_docode2(node *n, int flags)
     int adroppings , bdroppings;
     current_switch.jumptable=0;
 
-
     if(!CDDR(n))
     {
       tmp1=alloc_label();
@@ -1130,7 +1165,7 @@ static int do_docode2(node *n, int flags)
 	  match_types(CAR(n)->type, object_type_string)))
       {
 	code_expression(CDR(n), 0, "assignment");
-	emit0(F_LTOSVAL2);
+	emit0(F_LTOSVAL2_AND_FREE);
       }else{
 	emit0(F_LTOSVAL);
 	code_expression(CDR(n), 0, "assignment");
@@ -1222,9 +1257,9 @@ static int do_docode2(node *n, int flags)
 	  num_args = do_docode(CDAR(n), 0);
 	  switch (num_args)
 	  {
-	    case 0: emit0(F_LTOSVAL1); break;
-	    case 1: emit0(F_LTOSVAL2); break;
-	    case 2: emit0(F_LTOSVAL3); break;
+	    case 0: emit0(F_LTOSVAL_AND_FREE); break;
+	    case 1: emit0(F_LTOSVAL2_AND_FREE); break;
+	    case 2: emit0(F_LTOSVAL3_AND_FREE); break;
 #ifdef PIKE_DEBUG
 	    default:
 	      Pike_fatal("Arglebargle glop-glyf?\n");
@@ -1241,7 +1276,7 @@ static int do_docode2(node *n, int flags)
 	  emit0(CAR(n)->token);
 
 	emit0(n->token);
-	return n->token==F_ASSIGN;
+	return n->token==F_ASSIGN; /* So when is this false? /mast */
       }
     case F_APPLY:
       if ((CAAR(n)->token == F_CONSTANT) &&
@@ -1936,7 +1971,7 @@ static int do_docode2(node *n, int flags)
       tmp=findstring("call_function");
       if(!tmp) yyerror("No call_function efun.");
       foo=find_module_identifier(tmp,0);
-      if(!foo || !foo->token==F_CONSTANT)
+      if(!foo || foo->token!=F_CONSTANT)
       {
 	yyerror("No call_function efun.");
       }else{
@@ -2529,12 +2564,18 @@ static int do_docode2(node *n, int flags)
 #if 0
 	  struct object *o;
 
-	  for(o=Pike_compiler->fake_object;o!=n->u.sval.u.object;o=o->parent)
+	  for(o=Pike_compiler->fake_object;o!=n->u.sval.u.object;o=o->parent) {
+	    state->new_program->flags |=
+	      PROGRAM_USES_PARENT | PROGRAM_NEEDS_PARENT;
 	    x++;
+	  }
 #else
 	  struct program_state *state=Pike_compiler;
-	  for(;state->fake_object!=n->u.sval.u.object;state=state->previous)
+	  for(;state->fake_object!=n->u.sval.u.object;state=state->previous) {
+	    state->new_program->flags |=
+	      PROGRAM_USES_PARENT | PROGRAM_NEEDS_PARENT;
 	    x++;
+	  }
 #endif
 	  emit2(F_EXTERNAL, n->u.sval.subtype, x);
 	  Pike_compiler->new_program->flags |=
