@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: docode.h,v 1.19 2003/11/19 17:19:29 grubba Exp $
+|| $Id: ba67501341e7d67baa1d0d8a233f5137b02cb73f $
 */
 
 #ifndef DOCODE_H
@@ -17,9 +17,9 @@
 
 #define WANT_LVALUE (DO_LVALUE | DO_INDIRECT)
 
-#define emit0(X)     insert_opcode0((X),lex.current_line, lex.current_file)
-#define emit1(X,Y)   insert_opcode1((X),(Y),lex.current_line, lex.current_file)
-#define emit2(X,Y,Z) insert_opcode2((X),(Y),(Z),lex.current_line, lex.current_file)
+#define emit0(X)     insert_opcode0((X),c->lex.current_line, c->lex.current_file)
+#define emit1(X,Y)   insert_opcode1((X),(Y),c->lex.current_line, c->lex.current_file)
+#define emit2(X,Y,Z) insert_opcode2((X),(Y),(Z),c->lex.current_line, c->lex.current_file)
 
 /* Prototypes begin here */
 void upd_int(int offset, INT32 tmp);
@@ -29,6 +29,7 @@ void push_explicit(INT32 address);
 INT32 pop_address(void);
 int alloc_label(void);
 int do_jump(int token,INT32 lbl);
+void modify_stack_depth(int delta);
 void do_pop(int x);
 int do_docode(node *n, int flags);
 void do_cond_jump(node *n, int label, int iftrue, int flags);
