@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: gc.h,v 1.146 2010/06/22 09:24:32 mast Exp $
+|| $Id: dcfd5674f8bc8aaa759f7f27cc5f31fc8248f269 $
 */
 
 #ifndef GC_H
@@ -67,15 +67,15 @@ extern double gc_average_slowness;
 #ifdef INT64
 #define ALLOC_COUNT_TYPE INT64
 #define ALLOC_COUNT_TYPE_MAX MAX_INT64
-#define PRINT_ALLOC_COUNT_TYPE "%"PRINTINT64"d"
+#define PRINT_ALLOC_COUNT_TYPE PRINTINT64"d"
 #else
 #define ALLOC_COUNT_TYPE unsigned long
 #define ALLOC_COUNT_TYPE_MAX ULONG_MAX
-#define PRINT_ALLOC_COUNT_TYPE "%lu"
+#define PRINT_ALLOC_COUNT_TYPE "lu"
 #endif
 
 extern int num_objects, got_unlinked_things;
-extern ALLOC_COUNT_TYPE num_allocs, alloc_threshold;
+extern ALLOC_COUNT_TYPE num_allocs, alloc_threshold, saved_alloc_threshold;
 PMOD_EXPORT extern int Pike_in_gc;
 extern int gc_generation;
 extern int gc_trace, gc_debug;
@@ -291,6 +291,9 @@ extern size_t gc_ext_weak_refs;
 typedef void gc_cycle_check_cb (void *data, int weak);
 
 struct gc_frame;
+void count_memory_in_ba_mixed_frames(size_t *num, size_t *size);
+void count_memory_in_gc_rec_frames(size_t *num, size_t *size);
+void count_memory_in_mc_markers(size_t *num, size_t *size);
 struct callback *debug_add_gc_callback(callback_func call,
 				 void *arg,
 				 callback_func free_func);
