@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: interpret.h,v 1.181 2010/03/24 20:47:54 mast Exp $
+|| $Id: d065158974347947e3e6d8ec0f27d12a58deb2f0 $
 */
 
 #ifndef INTERPRET_H
@@ -97,7 +97,6 @@ struct pike_frame
 #if defined(PROFILING)
   cpu_time_t children_base;	/** Accounted time when the frame started. */
   cpu_time_t start_time;	/** Adjusted time when thr frame started. */
-  cpu_time_t self_time_base;	/* ??? */
 #endif
 };
 
@@ -609,10 +608,9 @@ PMOD_EXPORT extern const char msg_pop_neg[];
 		  });							\
       Pike_interpreter.accounted_time += self_time;			\
       /* FIXME: Can context->prog be NULL? */				\
-      function = _fp_->context->prog->identifiers + _fp_->ident;		\
-      /* function->total_time =						\
-	 Pike_fp->self_time_base + time_passed; */			\
-      function->total_time += time_passed;				\
+      function = _fp_->context->prog->identifiers + _fp_->ident;	\
+      if (!--function->recur_depth)					\
+	function->total_time += time_passed;				\
       function->self_time += self_time;					\
     });									\
   LOW_POP_PIKE_FRAME (_fp_);						\
