@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: lexer.h,v 1.48 2004/03/24 20:29:25 grubba Exp $
+|| $Id: 3a1596be7f0a88de70ff7dcdaeebfa0c2fbf06c4 $
 */
 
 /*
@@ -28,16 +28,19 @@
 
 #if (SHIFT == 0)
 
-#define LOOK() EXTRACT_UCHAR(lex.pos)
-#define GETC() EXTRACT_UCHAR(lex.pos++)
-#define SKIP() lex.pos++
+#define WCHAR p_wchar0
+
+#define LOOK() EXTRACT_UCHAR(lex->pos)
+#define GETC() EXTRACT_UCHAR(lex->pos++)
+#define SKIP() lex->pos++
+#define SKIPN(N) (lex->pos += (N))
 
 #define READBUF(X) do {				\
   register int C;				\
-  buf = lex.pos;				\
+  buf = lex->pos;				\
   while((C = LOOK()) && (X))			\
-    lex.pos++;					\
-  len = (size_t)(lex.pos - buf);		\
+    lex->pos++;					\
+  len = (size_t)(lex->pos - buf);		\
 } while(0)
 
 #define TWO_CHAR(X,Y) ((X)<<8)+(Y)
@@ -47,6 +50,7 @@
  * Function renaming
  */
 
+#define parse_esc_seq parse_esc_seq0
 #define yylex yylex0
 #define low_yylex low_yylex0
 #define lex_atoi atoi
@@ -56,26 +60,30 @@
 
 #else /* SHIFT != 0 */
 
-#define LOOK() INDEX_CHARP(lex.pos,0,SHIFT)
-#define SKIP() (lex.pos += (1<<SHIFT))
-#define GETC() (SKIP(),INDEX_CHARP(lex.pos-(1<<SHIFT),0,SHIFT))
+#define LOOK() INDEX_CHARP(lex->pos,0,SHIFT)
+#define SKIP() (lex->pos += (1<<SHIFT))
+#define SKIPN(N) (lex->pos += ((N)<<SHIFT))
+#define GETC() (SKIP(),INDEX_CHARP(lex->pos-(1<<SHIFT),0,SHIFT))
 
 #define READBUF(X) do {				\
   register int C;				\
-  buf = lex.pos;				\
+  buf = lex->pos;				\
   while((C = LOOK()) && (X))			\
     SKIP();					\
-  len = (size_t)((lex.pos - buf) >> SHIFT);	\
+  len = (size_t)((lex->pos - buf) >> SHIFT);	\
 } while(0)
 
 #define TWO_CHAR(X,Y) ((X)<<8)+(Y)
 
 #define ISWORD(X) ((len == strlen(X)) && low_isword(buf, X, strlen(X)))
 
-/* Function renaming */
 #if (SHIFT == 1)
 
+#define WCHAR p_wchar1
+
+/* Function renaming */
 #define low_isword low_isword1
+#define parse_esc_seq parse_esc_seq1
 #define char_const char_const1
 #define readstring readstring1
 #define yylex yylex1
@@ -86,7 +94,11 @@
 
 #else /* SHIFT != 1 */
 
+#define WCHAR p_wchar2
+
+/* Function renaming */
 #define low_isword low_isword2
+#define parse_esc_seq parse_esc_seq2
 #define char_const char_const2
 #define readstring readstring2
 #define yylex yylex2
@@ -154,7 +166,7 @@ static double lex_strtod(char *buf, char **end)
  *   \[0-7]*		octal escape
  *   \a			alert (BEL)
  *   \b			backspace (BS)
- *   \d[0-9]		decimal escape
+ *   \d[0-9]*		decimal escape
  *   \e			escape (ESC)
  *   \f			form-feed (FF)
  *   \n			newline (LF)
@@ -162,81 +174,186 @@ static double lex_strtod(char *buf, char **end)
  *   \t			tab (HT)
  *   \v			vertical-tab (VT)
  *   \x[0-9a-fA-F]*	hexadecimal escape
+ *   \u+[0-9a-fA-F]{4,4} 16 bit unicode style escape
+ *   \U+[0-9a-fA-F]{8,8} 32 bit unicode style escape
+ *
+ * If there are more than one u or U in the unicode style escapes, one
+ * is removed and the escape remains otherwise intact.
  */
-static int char_const(void)
-{
-  int c;
-  switch(c=GETC())
-  {
-    case 0:
-      lex.pos -= (1<<SHIFT);
-      yyerror("Unexpected end of file\n");
-      return 0;
-      
-    case '0': case '1': case '2': case '3':
-    case '4': case '5': case '6': case '7':
-      c-='0';
-      while(LOOK()>='0' && LOOK()<='8')
-	c=c*8+(GETC()-'0');
-      return c;
-      
-    case 'a': return 7;		/* BEL */
-    case 'b': return 8;		/* BS */
-    case 't': return 9;		/* HT */
-    case 'n': return 10;	/* LF */
-    case 'v': return 11;	/* VT */
-    case 'f': return 12;	/* FF */
-    case 'r': return 13;	/* CR */
-    case 'e': return 27;	/* ESC */
-      
-    case '\n':
-      lex.current_line++;
-      return '\n';
-      
-    case 'x':
-      c=0;
-      while(1)
-      {
-	switch(LOOK())
-	{
-	  case '0': case '1': case '2': case '3':
-	  case '4': case '5': case '6': case '7':
-	  case '8': case '9':
-	    c=c*16+GETC()-'0';
-	    continue;
-	    
-	  case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
-	    c=c*16+GETC()-'a'+10;
-	    continue;
-	    
-	  case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
-	    c=c*16+GETC()-'A'+10;
-	    continue;
-	}
-	break;
-      }
-      break;
 
-    case 'd':
-      c=0;
-      while(1)
-      {
-	switch(LOOK())
-	{
-	  case '0': case '1': case '2': case '3':
-	  case '4': case '5': case '6': case '7':
-	  case '8': case '9':
-	    c=c*10+GETC()-'0';
+int parse_esc_seq (WCHAR *buf, p_wchar2 *chr, ptrdiff_t *len)
+/* buf is assumed to be after the backslash. Return codes:
+ * 0: All ok. The char's in *chr, consumed length in *len.
+ * 1: Found a literal \r at *buf.
+ * 2: Found a literal \n at *buf.
+ * 3: Found a literal \0 at *buf.
+ * 4: Too large octal escape. *len is gobbled to the end of it all.
+ * 5: Too large hexadecimal escape. *len is gobbled to the end of it all.
+ * 6: Too large decimal escape. *len is gobbled to the end of it all.
+ * 7: Not 4 digits in \u escape. *len is up to the last found digit.
+ * 8: Not 8 digits in \U escape. *len is up to the last found digit. */
+{
+  ptrdiff_t l = 1;
+  p_wchar2 c;
+
+  switch ((c = *buf))
+  {
+    case '\r': return 1;
+    case '\n': return 2;
+    case 0: return 3;
+
+    case 'a': c = 7; break;	/* BEL */
+    case 'b': c = 8; break;	/* BS */
+    case 't': c = 9; break;	/* HT */
+    case 'n': c = 10; break;	/* LF */
+    case 'v': c = 11; break;	/* VT */
+    case 'f': c = 12; break;	/* FF */
+    case 'r': c = 13; break;	/* CR */
+    case 'e': c = 27; break;	/* ESC */
+
+    case '0': case '1': case '2': case '3':
+    case '4': case '5': case '6': case '7': {
+      unsigned of = 0;
+      unsigned INT32 n = c-'0';
+      for (l = 1; buf[l] >= '0' && buf[l] <= '8'; l++) {
+	if (!of) of = UNSIGNED_INT_TYPE_MUL_OVERFLOW (n, eight);
+	n = 8 * n + buf[l] - '0';
+      }
+      if (of) {*len = l; return 4;}
+      c = (p_wchar2)n;
+      break;
+    }
+      
+    case 'x': {
+      unsigned of = 0;
+      unsigned INT32 n=0;
+      for (l = 1;; l++) {
+	switch (buf[l]) {
+	  case '0': case '1': case '2': case '3': case '4':
+	  case '5': case '6': case '7': case '8': case '9':
+	    if (!of) of = UNSIGNED_INT_TYPE_MUL_OVERFLOW (n, sixteen);
+	    n = 16 * n + buf[l] - '0';
+	    continue;
+	  case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
+	    if (!of) of = UNSIGNED_INT_TYPE_MUL_OVERFLOW (n, sixteen);
+	    n = 16 * n + buf[l] - 'a' + 10;
+	    continue;
+	  case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
+	    if (!of) of = UNSIGNED_INT_TYPE_MUL_OVERFLOW (n, sixteen);
+	    n = 16 * n + buf[l] - 'A' + 10;
 	    continue;
 	}
 	break;
       }
+      if (of) {*len = l; return 5;}
+      c = (p_wchar2)n;
       break;
+    }
+
+    case 'd': {
+      unsigned of = 0;
+      unsigned INT32 n=0;
+      for (l = 1;; l++) {
+	switch (buf[l]) {
+	  case '0': case '1': case '2': case '3': case '4':
+	  case '5': case '6': case '7': case '8': case '9':
+	    if (!of) of = UNSIGNED_INT_TYPE_MUL_OVERFLOW (n, ten);
+	    n = 10 * n + buf[l] - '0';
+	    continue;
+	}
+	break;
+      }
+      if (of) {*len = l; return 6;}
+      c = (p_wchar2)n;
+      break;
+    }
+
+    case 'u':
+    case 'U': {
+      /* FIXME: Do we need compat goo to turn this off? */
+      /* Note: Code dup in gobble_identifier in preprocessor.h. */
+      int stop, longq;
+      l = 1;
+      if (buf[1] == c) {
+	/* A double-u quoted escape. Convert the "\u" or "\U" to "\",
+	 * thereby shaving off a "u" or "U" from the escape
+	 * sequence. */
+	/* Don't check that there's a valid number of hex digits in
+	 * this case, since the encoding code that can produce them
+	 * doesn't check that. */
+	c = '\\';
+	break;
+      }
+      if (c == 'u') {
+	stop = l + 4;
+	longq = 0;
+      }
+      else {
+	stop = l + 8;
+	longq = 1;
+      }
+      c = 0;
+      for (; l < stop; l++)
+	switch (buf[l]) {
+	  case '0': case '1': case '2': case '3': case '4':
+	  case '5': case '6': case '7': case '8': case '9':
+	    c = 16 * c + buf[l] - '0';
+	    break;
+	  case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
+	    c = 16 * c + buf[l] - 'a' + 10;
+	    break;
+	  case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
+	    c = 16 * c + buf[l] - 'A' + 10;
+	    break;
+	  default:
+	    *len = l;
+	    return longq ? 8 : 7;
+	}
+    }
   }
+
+  *len = l;
+  *chr = c;
+  return 0;
+}
+
+static p_wchar2 char_const(struct lex *lex)
+{
+  p_wchar2 c;
+  ptrdiff_t l;
+  switch (parse_esc_seq ((WCHAR *)lex->pos, &c, &l)) {
+    case 0:
+      break;
+    case 1:
+      SKIP();
+      return '\r';
+    case 2:
+      SKIP();
+      lex->current_line++;
+      return '\n';
+    case 3:
+      yyerror("Unexpected end of file.");
+      lex->pos -= (1<<SHIFT);
+      return 0;
+    case 4: case 5: case 6:
+      if( Pike_compiler->compiler_pass == 1 )
+        yywarning ("Too large character value in escape.");
+      c = -1;
+      break;
+    case 7:
+      if( Pike_compiler->compiler_pass == 1 )
+        yywarning ("Too few hex digits in \\u escape.");
+      return '\\';
+    case 8:
+      if( Pike_compiler->compiler_pass == 1 )
+        yywarning ("Too few hex digits in \\U escape.");
+      return '\\';
+  }
+  SKIPN (l);
   return c;
 }
 
-static struct pike_string *readstring(void)
+static struct pike_string *readstring(struct lex *lex)
 {
   int c;
   struct string_builder tmp;
@@ -263,17 +380,17 @@ static struct pike_string *readstring(void)
     switch(c=GETC())
     {
     case 0:
-      lex.pos -= (1<<SHIFT);
+      lex->pos -= (1<<SHIFT);
       yyerror("End of file in string.");
       break;
       
     case '\n':
-      lex.current_line++;
+      lex->current_line++;
       yyerror("Newline in string.");
       break;
       
     case '\\':
-      string_builder_putchar(&tmp,char_const());
+      string_builder_putchar(&tmp, char_const(lex));
       continue;
       
     case '"':
@@ -287,31 +404,31 @@ static struct pike_string *readstring(void)
     }
     break;
   }
-  return finish_string_builder(&tmp);
+  return dmalloc_touch(struct pike_string *, finish_string_builder(&tmp));
 }
 
 
 
-int yylex(YYSTYPE *yylval)
+static int low_yylex(struct lex *lex, YYSTYPE *);
+int yylex(struct lex *lex, YYSTYPE *yylval)
 #if LEXDEBUG>4
 {
   int t;
-  int low_yylex(YYSTYPE *);
 #if LEXDEBUG>8
   fprintf(stderr, "YYLEX:\n");
 #endif /* LEXDEBUG>8 */
 
-  t=low_yylex(yylval);
+  t=low_yylex(lex, yylval);
   if(t<256)
   {
     fprintf(stderr,"YYLEX: '%c' (%d) at %s:%d\n",t,t,lex.current_file->str,lex.current_line);
   }else{
-    fprintf(stderr,"YYLEX: %s (%d) at %s:%d\n",low_get_f_name(t,0),t,lex.current_file->str,lex.current_line);
+    fprintf(stderr,"YYLEX: token #%d at %s:%d\n",t,lex.current_file->str,lex.current_line);
   }
   return t;
 }
 
-static int low_yylex(YYSTYPE *yylval)
+static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 #endif /* LEXDEBUG>4 */
 {
   INT32 c;
@@ -331,12 +448,12 @@ static int low_yylex(YYSTYPE *yylval)
 
     if((c>'9') && lex_isidchar(c))
     {
-      lex.pos -= (1<<SHIFT);
+      lex->pos -= (1<<SHIFT);
       READBUF(lex_isidchar(C));
 
       PIKE_MEM_WO_RANGE (yylval, sizeof (YYSTYPE));
 
-      if(len>1 && len<10)
+      if(len>1 && len<16)
       {
 	/* NOTE: TWO_CHAR() will generate false positives with wide strings,
 	 * but that doesn't matter, since ISWORD() will fix it.
@@ -376,6 +493,12 @@ static int low_yylex(YYSTYPE *yylval)
 	case TWO_CHAR('e','x'):
 	  if(ISWORD("extern")) return TOK_EXTERN;
 	  break;
+#ifdef WITH_FACETS
+#warning "facet" keyword needs compat if enabled by default
+	case TWO_CHAR('f','a'):
+	  if(ISWORD("facet")) return TOK_FACET;
+	  break;
+#endif
 	case TWO_CHAR('f','i'):
 	  if(ISWORD("final")) return TOK_FINAL_ID;
 	  break;
@@ -465,6 +588,55 @@ static int low_yylex(YYSTYPE *yylval)
 	case TWO_CHAR('w','h'):
 	  if(ISWORD("while")) return TOK_WHILE;
 	  break;
+	case TWO_CHAR('_','_'):
+	  if (TEST_COMPAT(7,6)) break;
+	  if(len < 5) break;
+	  if(ISWORD("__attribute__"))
+	    return TOK_ATTRIBUTE_ID;
+	  if(ISWORD("__deprecated__"))
+	    return TOK_DEPRECATED_ID;
+	  if(ISWORD("__func__"))
+	    return TOK_FUNCTION_NAME;
+	  /* Allow triple (or more) underscore for the user, and make sure we
+	   * don't get false matches below for wide strings.
+	   */
+	  if((INDEX_CHARP(buf, 2, SHIFT) == '_') ||
+	     (INDEX_CHARP(buf, len-3, SHIFT) == '_') ||
+	     (INDEX_CHARP(buf, len-2, SHIFT) != '_') ||
+	     (INDEX_CHARP(buf, len-1, SHIFT) != '_') ||
+	     (INDEX_CHARP(buf, 0, SHIFT) != '_') ||
+	     (INDEX_CHARP(buf, 1, SHIFT) != '_')) break;
+	  {
+	    /* Double underscore before and after is reserved for keywords. */
+#if (SHIFT == 0)
+	    struct pike_string *tmp = make_shared_binary_string(buf, len);
+#else /* SHIFT != 0 */
+#if (SHIFT == 1)
+	    struct pike_string *tmp = make_shared_binary_string1((p_wchar1 *)buf,
+								 len);
+#else /* SHIFT != 1 */
+	    struct pike_string *tmp = make_shared_binary_string2((p_wchar2 *)buf,
+								 len);
+#endif /* SHIFT == 1 */
+#endif /* SHIFT == 0 */
+	    yylval->n=mkstrnode(tmp);
+	    /* - But only for lower case US-ASCII.
+	     * - Upper case is used for symbols intended for #if constant().
+	     */
+	    if (tmp->size_shift) {
+	      free_string(tmp);	    
+	      return TOK_IDENTIFIER;
+	    }
+	    while(len--) {
+	      int c = tmp->str[len];
+	      if ((c >= 'A') && (c <= 'Z')) {
+		free_string(tmp);
+		return TOK_IDENTIFIER;
+	      }
+	    }
+	    free_string(tmp);	    
+	  }
+	  return TOK_RESERVED;
 	}
       }
       {
@@ -490,8 +662,8 @@ static int low_yylex(YYSTYPE *yylval)
     switch(c)
     {
     case 0:
-      lex.pos -= (1<<SHIFT);
-      if(lex.end != lex.pos)
+      lex->pos -= (1<<SHIFT);
+      if(lex->end != lex->pos)
 	yyerror("Illegal character (NUL)");
 
 #ifdef TOK_LEX_EOF
@@ -501,7 +673,7 @@ static int low_yylex(YYSTYPE *yylval)
 #endif /* TOK_LEX_EOF */
 
     case '\n':
-      lex.current_line++;
+      lex->current_line++;
       continue;
 
     case 0x1b: case 0x9b:	/* ESC or CSI */
@@ -529,22 +701,22 @@ static int low_yylex(YYSTYPE *yylval)
       {
       case '0': case '1': case '2': case '3': case '4':
       case '5': case '6': case '7': case '8': case '9':
-	lex.current_line=lex_atoi(buf)-1;
+	lex->current_line=lex_atoi(buf)-1;
 	SKIPSPACE();
 	if(GOBBLE('"'))
 	{
-	  struct pike_string *tmp=readstring();
-	  free_string(lex.current_file);
-	  lex.current_file=tmp;
+	  struct pike_string *tmp=readstring(lex);
+	  free_string(lex->current_file);
+	  lex->current_file = dmalloc_touch(struct pike_string *, tmp);
 	}
 	if (Pike_compiler->compiler_pass == 1 &&
 	    !Pike_compiler->new_program->num_linenumbers) {
 	  /* A nested program will always get an entry right away in
 	   * language.yacc. */
-	  store_linenumber(0, lex.current_file);
+	  store_linenumber(0, lex->current_file);
 #ifdef DEBUG_MALLOC
-	  if(strcmp(lex.current_file->str,"-"))
-	    debug_malloc_name(Pike_compiler->new_program, lex.current_file->str, 0);
+	  if(strcmp(lex->current_file->str,"-"))
+	    debug_malloc_name(Pike_compiler->new_program, lex->current_file->str, 0);
 #endif
 	}
 	break;
@@ -553,27 +725,40 @@ static int low_yylex(YYSTYPE *yylval)
 	if(ISWORD("pragma"))
 	{
 	  SKIPSPACE();
-	  READBUF(C!='\n');
+	  READBUF(C!='\n'&&C!=' ');
 	  if (ISWORD("all_inline"))
 	  {
-	    lex.pragmas |= ID_INLINE;
+	    lex->pragmas |= ID_INLINE;
 	  }
-	  else if (ISWORD("all_nomask"))
+	  else if (ISWORD("all_final") || ISWORD("all_nomask"))
 	  {
-	    lex.pragmas |= ID_NOMASK;
+	    lex->pragmas |= ID_FINAL;
 	  }
 	  else if (ISWORD("strict_types"))
 	  {
-	    lex.pragmas |= ID_STRICT_TYPES;
+	    lex->pragmas |= ID_STRICT_TYPES;
 	  }
 	  else if (ISWORD("save_parent"))
 	  {
-	    lex.pragmas |= ID_SAVE_PARENT;
+	    lex->pragmas |= ID_SAVE_PARENT;
 	  }
 	  else if (ISWORD("dont_save_parent"))
 	  {
-	    lex.pragmas |= ID_DONT_SAVE_PARENT;
+	    lex->pragmas |= ID_DONT_SAVE_PARENT;
 	  }
+	  else if (ISWORD("no_deprecation_warnings"))
+	  {
+	    lex->pragmas |= ID_NO_DEPRECATION_WARNINGS;
+	  }
+	  else if (ISWORD("deprecation_warnings"))
+	  {
+	    lex->pragmas &= ~ID_NO_DEPRECATION_WARNINGS;
+	  }
+          else
+          {
+            if( Pike_compiler->compiler_pass == 1 )
+              yywarning("Unknown #pragma directive.");
+          }
 	  break;
 	}
 
@@ -625,14 +810,8 @@ static int low_yylex(YYSTYPE *yylval)
 #error Unsupported SHIFT.
 #endif /* SHIFT == 2 */
 #endif /* SHIFT == 1 */
-	  if (!dir->size_shift) {
 #endif /* SHIFT == 0 */
-	    my_yyerror("Unknown preprocessor directive #%s.", dir->str);
-#if (SHIFT != 0)
-	  } else {
-	    yyerror("Unknown preprocessor directive.");
-	  }
-#endif /* SHIFT != 0 */
+	  my_yyerror("Unknown preprocessor directive %S.", dir);
 	  free_string(dir);
 	} else {
 	  yyerror("Unknown preprocessor directive.");
@@ -650,12 +829,12 @@ static int low_yylex(YYSTYPE *yylval)
       switch(c=GETC())
       {
       case 0:
-	lex.pos -= (1<<SHIFT);
+	lex->pos -= (1<<SHIFT);
 	yyerror("Unexpected end of file\n");
 	break;
 
       case '\\':
-	c = char_const();
+	c = char_const(lex);
 	break;
 
       case '\'':
@@ -670,7 +849,7 @@ static int low_yylex(YYSTYPE *yylval)
 	
     case '"':
     {
-      struct pike_string *s=readstring();
+      struct pike_string *s=readstring(lex);
       yylval->n=mkstrnode(s);
       free_string(s);
       return TOK_STRING;
@@ -686,7 +865,13 @@ static int low_yylex(YYSTYPE *yylval)
 	if(GOBBLE('.')) return TOK_DOT_DOT_DOT;
 	return TOK_DOT_DOT;
       }
-      return c;
+      if (((c = INDEX_CHARP(lex->pos, 0, SHIFT)) <= '9') &&
+	  (c >= '0')) {
+	/* FIXME: Only in Pike 7.7 and later mode? */
+	lex->pos -= (1<<SHIFT);
+	goto read_float;
+      }
+      return '.';
   
     case '0':
     {
@@ -705,12 +890,12 @@ static int low_yylex(YYSTYPE *yylval)
 	sval.type = PIKE_T_INT;
 	sval.subtype = NUMBER_NUMBER;
 	sval.u.integer = 0;
-	wide_string_to_svalue_inumber(&sval,
-				      lex.pos,
-				      (void **)&lex.pos,
-				      base,
-				      0,
-				      SHIFT);
+	safe_wide_string_to_svalue_inumber(&sval,
+					   lex->pos,
+					   &lex->pos,
+					   base,
+					   0,
+					   SHIFT);
 	dmalloc_touch_svalue(&sval);
 	yylval->n = mksvaluenode(&sval);
 	free_svalue(&sval);
@@ -723,55 +908,82 @@ static int low_yylex(YYSTYPE *yylval)
     {
       char *p1, *p2;
       double f;
-      long l = 0;
+      long l = 0;		/* GCC thinks l is unitialized here. Hmm..? */
       struct svalue sval;
 
-      lex.pos -= (1<<SHIFT);
-      if(INDEX_CHARP(lex.pos, 0, SHIFT)=='0')
-	for(l=1;INDEX_CHARP(lex.pos, l, SHIFT)<='9' &&
-	      INDEX_CHARP(lex.pos, l, SHIFT)>='0';l++)
-	  if(INDEX_CHARP(lex.pos, l, SHIFT)>='8')
+      lex->pos -= (1<<SHIFT);
+      if(INDEX_CHARP(lex->pos, 0, SHIFT)=='0')
+	for(l=1;INDEX_CHARP(lex->pos, l, SHIFT)<='9' &&
+	      INDEX_CHARP(lex->pos, l, SHIFT)>='0';l++)
+	  if(INDEX_CHARP(lex->pos, l, SHIFT)>='8')
 	    my_yyerror("Illegal octal digit '%c'.",
-		       INDEX_CHARP(lex.pos, l, SHIFT));
+		       INDEX_CHARP(lex->pos, l, SHIFT));
 
-      f=lex_strtod(lex.pos, &p1);
+    read_float:
+      f=lex_strtod(lex->pos, &p1);
 
       sval.type = PIKE_T_INT;
       sval.subtype = NUMBER_NUMBER;
       sval.u.integer = 0;      
 
-      wide_string_to_svalue_inumber(&sval,
-				    lex.pos,
-				    &p2,
-				    0,
-				    0,
-				    SHIFT);
-      dmalloc_touch_svalue(&sval);
-      yylval->n = mksvaluenode(&sval);
-      free_svalue(&sval);
-
+      safe_wide_string_to_svalue_inumber(&sval,
+					 lex->pos,
+					 &p2,
+					 0,
+					 0,
+					 SHIFT);
       if(p1>p2)
       {
-	debug_malloc_touch(yylval->n);
-	free_node(yylval->n);
+	/* Floating point or version. */
+	if ((sval.type == PIKE_T_INT) &&
+	    (INDEX_CHARP(p2, 0, SHIFT) == '.')) {
+	  int major = sval.u.integer;
+	  char *p3 = p2;
+	  p2 += (1<<SHIFT);
+	  dmalloc_touch_svalue(&sval);
+
+	  sval.u.integer = 0;
+	  safe_wide_string_to_svalue_inumber(&sval,
+					     p2,
+					     &p3,
+					     0,
+					     0,
+					     SHIFT);
+	  dmalloc_touch_svalue(&sval);
+	  if ((sval.type == PIKE_T_INT) && (p3 > p2)) {
+	    for (l=0; ISSPACE(INDEX_CHARP(p3, l, SHIFT)); l++)
+	      ;
+	    if ((INDEX_CHARP(p3, l, SHIFT) == ':') &&
+		(INDEX_CHARP(p3, l+1, SHIFT) == ':')) {
+	      /* Version prefix. */
+	      lex->pos = p3;
+	      yylval->n = mkversionnode(major, sval.u.integer);
+	      return TOK_VERSION;
+	    }
+	  }
+	}
+	free_svalue(&sval);
 	yylval->fnum=(FLOAT_TYPE)f;
 #if 0
 	fprintf(stderr, "LEX: \"%.8s\" => %f, %f\n",
-		(char *)lex.pos, f, yylval->fnum);
+		(char *)lex->pos, f, yylval->fnum);
 #endif /* 0 */
-	lex.pos=p1;
+	lex->pos=p1;
 	if (lex_isidchar (LOOK())) {
 	  my_yyerror ("Invalid char '%c' in constant.",
-		      INDEX_CHARP (lex.pos, l, SHIFT));
+		      INDEX_CHARP (lex->pos, l, SHIFT));
 	  do SKIP(); while (lex_isidchar (LOOK()));
 	}
 	return TOK_FLOAT;
       }else{
+	dmalloc_touch_svalue(&sval);
+	yylval->n = mksvaluenode(&sval);
+	free_svalue(&sval);
 	debug_malloc_touch(yylval->n);
-	lex.pos=p2;
+	lex->pos=p2;
 	if (lex_isidchar (LOOK())) {
 	  my_yyerror ("Invalid char '%c' in constant.",
-		      INDEX_CHARP (lex.pos, l, SHIFT));
+		      INDEX_CHARP (lex->pos, l, SHIFT));
 	  do SKIP(); while (lex_isidchar (LOOK()));
 	}
 	return TOK_NUMBER;
@@ -868,7 +1080,7 @@ static int low_yylex(YYSTYPE *yylval)
 	}
       }
       
-      switch(GETC())
+      switch(c = GETC())
       {
       case '/': tmp="```/"; break;
       case '%': tmp="```%"; break;
@@ -915,6 +1127,35 @@ static int low_yylex(YYSTYPE *yylval)
       case '-':
 	if(GOBBLE('>'))
 	{
+	  if ((offset == 2) && lex_isidchar(LOOK())) {
+	    /* Getter/setter (old-style)
+	     *
+	     * Either
+	     *   `->symbol
+	     * Or
+	     *   `->symbol=
+	     */
+	    char *buf;
+	    size_t len;
+	    struct pike_string *s;
+	    READBUF(lex_isidchar(C));
+	    if (GOBBLE('=')) len += 1;
+	    /* Adjust for the prefix (`->). */
+	    len += 3;
+	    buf -= 3<<SHIFT;
+#if (SHIFT == 0)
+	    s = make_shared_binary_string(buf, len);
+#else /* SHIFT != 0 */
+#if (SHIFT == 1)
+	    s = make_shared_binary_string1((p_wchar1 *)buf, len);
+#else /* SHIFT != 1 */
+	    s = make_shared_binary_string2((p_wchar2 *)buf, len);
+#endif /* SHIFT == 1 */
+#endif /* SHIFT == 0 */
+	    yylval->n = mkstrnode(s);
+	    free_string(s);
+	    return TOK_IDENTIFIER;
+	  }
 	  tmp="```->";
 	  if(GOBBLE('=')) tmp="```->=";
 	}else{
@@ -929,12 +1170,45 @@ static int low_yylex(YYSTYPE *yylval)
 	  if(GOBBLE('=')) tmp="```[]=";
 	  break;
 	}
-	yyerror("Illegal ` identifier. Expected `[].");
+	if (GOBBLE ('.') && GOBBLE ('.') && GOBBLE (']')) {
+	  tmp = "```[..]";
+	  break;
+	}
+	yyerror("Illegal ` identifier. Expected `[], `[]= or `[..].");
 	break;
 
       default:
+	  if (offset==2 && lex_isidchar(c)) {
+	    /* Getter/setter (new-style)
+	     *
+	     * Either
+	     *   `symbol
+	     * Or
+	     *   `symbol=
+	     */
+	    char *buf;
+	    size_t len;
+	    struct pike_string *s;
+	    READBUF(lex_isidchar(C));
+	    if (GOBBLE('=')) len += 1;
+	    /* Adjust for the prefix (`c). */
+	    len += 2;
+	    buf -= 2<<SHIFT;
+#if (SHIFT == 0)
+	    s = make_shared_binary_string(buf, len);
+#else /* SHIFT != 0 */
+#if (SHIFT == 1)
+	    s = make_shared_binary_string1((p_wchar1 *)buf, len);
+#else /* SHIFT != 1 */
+	    s = make_shared_binary_string2((p_wchar2 *)buf, len);
+#endif /* SHIFT == 1 */
+#endif /* SHIFT == 0 */
+	    yylval->n = mkstrnode(s);
+	    free_string(s);
+	    return TOK_IDENTIFIER;
+	  }
 	yyerror("Illegal ` identifier.");
-	lex.pos -= (1<<SHIFT);
+	lex->pos -= (1<<SHIFT);
 	tmp="```";
 	break;
       }
@@ -951,9 +1225,9 @@ static int low_yylex(YYSTYPE *yylval)
     default:
       {
 	if (c > 31) {
-	  my_yyerror("Illegal character (hex %02x) '%c'", c, c);
+	  my_yyerror("Illegal character '%c' (0x%02x)", c, c);
 	} else {
-	  my_yyerror("Illegal character (hex %02x)", c);
+	  my_yyerror("Illegal character 0x%02x", c);
 	}
 	return ' ';
       }
@@ -965,9 +1239,11 @@ static int low_yylex(YYSTYPE *yylval)
  * Clear the defines for the next pass
  */
 
+#undef WCHAR
 #undef LOOK
 #undef GETC
 #undef SKIP
+#undef SKIPN
 #undef GOBBLE
 #undef SKIPSPACE
 #undef SKIPWHITE
@@ -977,6 +1253,7 @@ static int low_yylex(YYSTYPE *yylval)
 #undef ISWORD
 
 #undef low_isword
+#undef parse_esc_seq
 #undef char_const
 #undef readstring
 #undef yylex
