@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: png.c,v 1.101 2010/05/29 16:12:12 nilsson Exp $
+|| $Id: 41abf82ed30d735763ecf11325617857a34ef6fd $
 */
 
 #include "global.h"
@@ -631,7 +631,8 @@ static int _png_write_rgb(rgb_group *w1,
 	    rgb_group tr;
 	    tr.r=tr.g=tr.b=_png_c16(int_from_16bit(trns->str),bpp);
 	    n=n0;
-	    while (n--) *(da1++)=tr;
+	    d1=w1;
+	    while (n--) *(da1++)=(tr.r==(d1++)->r)?black:white;
 	    return 1; /* alpha channel */
 	 }
 	 return 0; /* no alpha channel */
@@ -676,7 +677,8 @@ static int _png_write_rgb(rgb_group *w1,
 	    tr.b=_png_c16(int_from_16bit(trns->str+4),bpp);
 
 	    n=n0;
-	    while (n--) *(da1++)=tr;
+	    d1=w1;
+	    while (n--) *(da1++)=(tr.r==d1->r && tr.g==d1->g && tr.b==d1->b)?black:white,d1++;
 	    return 1; /* alpha channel */
 	 }
 	 return 0; /* no alpha channel */
