@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: x.c,v 1.51 2008/06/16 21:56:24 mast Exp $
+|| $Id: bd7787e34836b1f2156266697589243fb9e8eece $
 */
 
 /*
@@ -587,7 +587,7 @@ static void image_x_encode_pseudocolor_1byte(INT32 args,
 	 x=img->xsize; 
 	 while (x--) 
 	 {
-	    b = (translate[*(s++)]<<(32-vbpp)) && 0xffffffff;
+	    b = (translate[*(s++)]<<(32-vbpp)) & 0xffffffff;
 	    bp = bpp;
 	    while (bp>8-bit)
 	    {
