@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: e5b53a484c0e5a4526938df926324be6e8eb192f $
+|| $Id: 9ce5334b8cb3503385a52b9c3ac6b9a8e9d8a717 $
 */
 
 /*
@@ -26,6 +26,9 @@
 /* Return type of mysql_fetch_lengths(). Usually unsigned long. */
 #undef FETCH_LENGTHS_TYPE
 
+/* Define if your mysql.h defines MYSQL_OPT_RECONNECT */
+#undef HAVE_MYSQL_OPT_RECONNECT
+
 /* Define if your mysql.h defines MYSQL_OPT_CONNECT_TIMEOUT */
 #undef HAVE_MYSQL_OPT_CONNECT_TIMEOUT
 
@@ -59,4 +62,7 @@
 /* Define if MYSQL_FIELD has a charsetnr member */
 #undef HAVE_MYSQL_FIELD_CHARSETNR
 
+/* Define if MYSQL has a net.vio member */
+#undef HAVE_MYSQL_FIELD_NET_VIO
+
 #endif /* PIKE_MYSQL_CONFIG_H */
