@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 44d8150458f480f9d5b37fd16d16c3dfa737b7b2 $
+|| $Id: 7ae5ae97a9bb16ce63bbb2ac62913735eac8c780 $
 */
 
 /*
@@ -87,6 +87,10 @@
 #include <memory.h>
 #endif
 
+#ifdef HAVE_POLL_H
+#include <poll.h>
+#endif
+
 #define sp Pike_sp
 
 #ifdef HAVE_MYSQL
@@ -287,6 +291,15 @@ static void pike_mysql_set_options(struct mapping *options)
   struct svalue *val;
 
 #ifdef HAVE_MYSQL_OPTIONS
+  if ((val = simple_mapping_string_lookup(options, "reconnect"))) {
+    my_bool reconnectp = 0;
+    if (!SAFE_IS_ZERO(val)) reconnectp = 1;
+#ifdef HAVE_MYSQL_OPT_RECONNECT
+    mysql_options(PIKE_MYSQL->mysql, MYSQL_OPT_RECONNECT, &reconnectp);
+#else
+    PIKE_MYSQL->mysql->reconnect = reconnectp;
+#endif
+  }
 #ifdef HAVE_MYSQL_READ_DEFAULT_FILE
   if ((val = simple_mapping_string_lookup(options, "mysql_config_file")) &&
       (val->type == T_STRING) && (!val->u.string->size_shift)) {
@@ -353,6 +366,11 @@ static void pike_mysql_set_options(struct mapping *options)
   if ((val = simple_mapping_string_lookup(options, "mysql_charset_name")) &&
       (val->type == T_STRING) && (!val->u.string->size_shift) &&
       !string_has_null (val->u.string)) {
+#if defined (HAVE_MYSQL_OPTIONS) && defined (HAVE_MYSQL_SET_CHARSET_NAME)
+    mysql_options (PIKE_MYSQL->mysql, MYSQL_SET_CHARSET_NAME,
+		   val->u.string->str);
+#endif
+
     if (PIKE_MYSQL->conn_charset)
       free (PIKE_MYSQL->conn_charset);
     copy_shared_string (PIKE_MYSQL->conn_charset, val->u.string);
@@ -361,12 +379,18 @@ static void pike_mysql_set_options(struct mapping *options)
 
 static void low_query(INT32 args, char *name, int flags);
 
-static void connection_set_charset()
+static void connection_set_charset (struct pike_string *charset)
 {
-#ifdef HAVE_MYSQL_SET_CHARACTER_SET
   int res;
   MYSQL *mysql = PIKE_MYSQL->mysql;
-  struct pike_string *charset = PIKE_MYSQL->conn_charset;
+
+#if defined (HAVE_MYSQL_OPTIONS) && defined (HAVE_MYSQL_SET_CHARSET_NAME)
+  /* Update the default charset for the connection,
+   * so that it will be restored on reconnect. */
+  mysql_options (mysql, MYSQL_SET_CHARSET_NAME, charset->str);
+#endif
+
+#ifdef HAVE_MYSQL_SET_CHARACTER_SET
   MYSQL_ALLOW();
   res = mysql_set_character_set (mysql, charset->str);
   MYSQL_DISALLOW();
@@ -384,7 +408,7 @@ static void connection_set_charset()
    * that this doesn't work with mysql_real_escape_string, but that
    * function isn't used. */
   push_constant_text ("SET NAMES '");
-  ref_push_string (PIKE_MYSQL->conn_charset);
+  ref_push_string (charset);	/* FIXME: Quote? */
   push_constant_text ("'");
   f_add (3);
   low_query (1, "set_charset", PIKE_MYSQL_FLAG_STORE_RESULT);
@@ -439,6 +463,14 @@ static void pike_mysql_reconnect (int reconnect)
     options = (unsigned int)val->u.integer;
   }
 
+#if defined(HAVE_MYSQL_OPTIONS) && defined(HAVE_MYSQL_OPT_RECONNECT)
+  {
+    /* Enable the automatic reconnect. */
+    my_bool reconnectp = 1;
+    mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnectp);
+  }
+#endif
+
 #if defined (HAVE_MYSQL_OPTIONS) && defined (HAVE_MYSQL_SET_CHARSET_NAME)
   if (PIKE_MYSQL->conn_charset)
     mysql_options (mysql, MYSQL_SET_CHARSET_NAME,
@@ -448,6 +480,10 @@ static void pike_mysql_reconnect (int reconnect)
 
   MYSQL_ALLOW();
 
+#ifdef HAVE_MYSQL_REAL_CONNECT
+  socket = mysql_real_connect(mysql, host, user, password,
+                              NULL, port, portptr, options);
+#else
 #if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
   STUPID_PORT_LOCK();
 #endif /* HAVE_MYSQL_PORT || HAVE_MYSQL_UNIX_PORT */
@@ -465,12 +501,7 @@ static void pike_mysql_reconnect (int reconnect)
   }
 #endif /* HAVE_MYSQL_UNIX_PORT */
 
-#ifdef HAVE_MYSQL_REAL_CONNECT
-  socket = mysql_real_connect(mysql, host, user, password,
-                              NULL, port, portptr, options);
-#else
   socket = mysql_connect(mysql, host, user, password);
-#endif /* HAVE_MYSQL_REAL_CONNECT */
 
 #ifdef HAVE_MYSQL_PORT
   if (port) {
@@ -482,12 +513,26 @@ static void pike_mysql_reconnect (int reconnect)
     mysql_unix_port = saved_unix_port;
   }
 #endif /* HAVE_MYSQL_UNIX_PORT */
+
 #if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
   STUPID_PORT_UNLOCK();
 #endif /* HAVE_MYSQL_PORT || MAVE_MYSQL_UNIX_PORT*/
+#endif /* HAVE_MYSQL_REAL_CONNECT */
 
   MYSQL_DISALLOW();
 
+#ifndef HAVE_MYSQL_OPT_RECONNECT
+  /* Note: In Mysql 3.22 the reconnect flag is always set by
+   *       mysql_real_connect(), so we need to reset it here.
+   */
+  if (PIKE_MYSQL->options &&
+      (val = simple_mapping_string_lookup(PIKE_MYSQL->options, "reconnect"))) {
+    my_bool reconnectp = 0;
+    if (!SAFE_IS_ZERO(val)) reconnectp = 1;
+    PIKE_MYSQL->mysql->reconnect = reconnectp;
+  }
+#endif
+
   if (hostptr) {
     /* No longer needed */
     free(hostptr);
@@ -527,7 +572,7 @@ static void pike_mysql_reconnect (int reconnect)
 
 #ifndef RECONNECT_CHARSET_IS_SET
   if (PIKE_MYSQL->conn_charset)
-    connection_set_charset();
+    connection_set_charset (PIKE_MYSQL->conn_charset);
 #endif
 }
 
@@ -669,10 +714,13 @@ static void f_create(INT32 args)
 	  }
 	  if (args >= 5) {
 	    if (sp[4-args].type != T_MAPPING) {
-	      SIMPLE_BAD_ARG_ERROR ("Mysql.mysql->create", 5, "mapping(string:mixed)");
+	      if (!UNSAFE_IS_ZERO(sp + 4 - args)) {
+		SIMPLE_BAD_ARG_ERROR ("Mysql.mysql->create", 5, "mapping(string:mixed)");
+	      }
+	    } else {
+	      add_ref(PIKE_MYSQL->options = sp[4-args].u.mapping);
+	      pike_mysql_set_options(sp[4-args].u.mapping);
 	    }
-	    add_ref(PIKE_MYSQL->options = sp[4-args].u.mapping);
-	    pike_mysql_set_options(sp[4-args].u.mapping);
 	  }
 	}
       }
@@ -681,6 +729,11 @@ static void f_create(INT32 args)
 
   pike_mysql_set_ssl(PIKE_MYSQL->options);
 
+  if (!PIKE_MYSQL->conn_charset) {
+    /* Backward compat with Mysql 4.0 and earlier. */
+    PIKE_MYSQL->conn_charset = make_shared_string("latin1");
+  }
+
   pike_mysql_reconnect (0);
 
 #ifdef HAVE_MYSQL_CHARACTER_SET_NAME
@@ -752,6 +805,88 @@ static void mysql__sprintf(INT32 args)
   Pike_sp[-1].subtype = 1;
 }
 
+/*! @decl int is_open()
+ *!
+ *! Returns true if the connection seems to be open.
+ *!
+ *! @note
+ *!   This function only checks that there's an
+ *!   open connection, and that the other end hasn't
+ *!   closed it yet. No data is sent over the
+ *!   connection.
+ *!
+ *!   For a more reliable check of whether the connection
+ *!   is alive, please use @[ping()].
+ *!
+ *! @seealso
+ *!   @[ping()]
+ */
+static void f_is_open(INT32 args)
+{
+  int fd = PIKE_MYSQL->mysql->net.fd;
+
+  pop_n_elems(args);
+  if (
+#ifdef HAVE_MYSQL_FIELD_NET_VIO
+      !(PIKE_MYSQL->mysql->net.vio) ||
+#endif
+      (fd < 0)) {
+    push_int(0);
+  } else {
+#ifdef HAVE_POLL
+    struct pollfd fds[1];
+    fds->fd = fd;
+    fds->events = POLLOUT|POLLHUP;
+    fds->revents = 0;
+    if ((poll(fds, 1, 0) == 1) && (fds->revents & POLLOUT)) {
+      push_int(1);
+    } else {
+      /* POLLHUP or POLLERR or timeout */
+      push_int(0);
+    }
+#else
+    /* FIXME: Use select(). */
+    push_int(1);
+#endif
+  }
+}
+
+/*! @decl int ping()
+ *!
+ *! Check whether the connection is alive.
+ *!
+ *! @returns
+ *!   Returns one of the following:
+ *!   @int
+ *!     @value 0
+ *!       Everything ok.
+ *!     @value 1
+ *!       The connection reconnected automatically.
+ *!     @value -1
+ *!       The server has gone away, and the connection is dead.
+ *!   @endint
+ *!
+ *! @seealso
+ *!   @[is_open()]
+ */
+static void f_ping(INT32 args)
+{
+  MYSQL *mysql = PIKE_MYSQL->mysql;
+  unsigned long orig_id = mysql_thread_id(mysql);
+  int err;
+  MYSQL_ALLOW();
+  err = mysql_ping(mysql);
+  MYSQL_DISALLOW();
+  pop_n_elems(args);
+  if (err) {
+    push_int(-1);
+  } else if (orig_id != mysql_thread_id(mysql)) {
+    push_int(1);
+  } else {
+    push_int(0);
+  }
+}
+
 /*! @decl int affected_rows()
  *!
  *! Returns the number of rows affected by the last query.
@@ -761,9 +896,6 @@ static void f_affected_rows(INT32 args)
   MYSQL *mysql;
   INT64 count;
 
-  if (!PIKE_MYSQL->mysql) {
-    pike_mysql_reconnect (1);
-  }
   pop_n_elems(args);
   mysql = PIKE_MYSQL->mysql;
 
@@ -784,9 +916,6 @@ static void f_insert_id(INT32 args)
   MYSQL *mysql;
   INT64 id;
 
-  if (!PIKE_MYSQL->mysql) {
-    pike_mysql_reconnect (1);
-  }
   pop_n_elems(args);
 
   mysql = PIKE_MYSQL->mysql;
@@ -809,10 +938,6 @@ static void f_error(INT32 args)
   MYSQL *mysql;
   const char *error_msg;
 
-  if (!PIKE_MYSQL->mysql) {
-    pike_mysql_reconnect (1);
-  }
-
   mysql = PIKE_MYSQL->mysql;
 
   MYSQL_ALLOW();
@@ -860,18 +985,6 @@ static void f_select_db(INT32 args)
 
     MYSQL_DISALLOW();
   }
-  if (!mysql || tmp) {
-    /* The connection might have been closed. */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-
-    tmp = mysql_select_db(mysql, database);
-
-    MYSQL_DISALLOW();
-  }
 
   if (tmp) {
     const char *err;
@@ -920,44 +1033,6 @@ static void low_query(INT32 args, char *name, int flags)
   if (mysql) {
     MYSQL_ALLOW();
 
-#ifdef HAVE_MYSQL_REAL_QUERY
-    tmp = mysql_real_query(mysql, query, qlen);
-#else
-    tmp = mysql_query(mysql, query);
-#endif /* HAVE_MYSQL_REAL_QUERY */
-
-    if (!tmp) {
-      if (flags & PIKE_MYSQL_FLAG_STORE_RESULT) {
-	result = mysql_store_result(mysql);
-      } else {
-	result = mysql_use_result(mysql);
-      }
-    }
-
-    MYSQL_DISALLOW();
-  }
-  if (mysql && tmp) {
-    /* Check if we need to reconnect. */
-#if defined(CR_SERVER_GONE_ERROR) && defined(CR_SERVER_LOST) && \
-  defined(CR_UNKNOWN_ERROR)
-    int eno = mysql_errno(mysql);
-    if ((eno == CR_SERVER_GONE_ERROR) ||
-	(eno == CR_SERVER_LOST) ||
-	(eno == CR_UNKNOWN_ERROR)) {
-      mysql = NULL;
-    }
-#else /* !CR_SERVER_GONE_ERROR || !CR_SERVER_LOST || !CR_UNKNOWN_ERROR */
-    mysql = NULL;
-#endif /* CR_SERVER_GONE_ERROR && CR_SERVER_LOST && CR_UNKNOWN_ERROR */
-  }
-  if (!mysql) {
-    /* The connection might have been closed. */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-
 #ifdef HAVE_MYSQL_REAL_QUERY
     tmp = mysql_real_query(mysql, query, qlen);
 #else
@@ -1104,18 +1179,6 @@ static void f_create_db(INT32 args)
     tmp = mysql_create_db(mysql, database);
     MYSQL_DISALLOW();
   }
-  if (!mysql || tmp) {
-    /* The connection might have been closed */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-
-    tmp = mysql_create_db(mysql, database);
-
-    MYSQL_DISALLOW();
-  }
 
   if (tmp) {
     Pike_error("Mysql.mysql->create_db(): Creation of database \"%s\" failed\n",
@@ -1162,18 +1225,6 @@ static void f_drop_db(INT32 args)
 
     MYSQL_DISALLOW();
   }
-  if (!mysql || tmp) {
-    /* The connection might have been closed */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-
-    tmp = mysql_drop_db(mysql, database);
-
-    MYSQL_DISALLOW();
-  }    
 
   if (tmp) {
     Pike_error("Mysql.mysql->drop_db(): Drop of database \"%s\" failed\n",
@@ -1201,23 +1252,6 @@ static void f_shutdown(INT32 args)
   if (mysql) {
     MYSQL_ALLOW();
   
-#ifdef HAVE_SHUTDOWN_DEFAULT
-    /* Mysql 4.1.3 added an extra shutdown_level argument. */
-    tmp = mysql_shutdown(mysql, SHUTDOWN_DEFAULT);
-#else /* !HAVE_SHUTDOWN_DEFAULT */
-    tmp = mysql_shutdown(mysql);
-#endif /* HAVE_SHUTDOWN_DEFAULT */
-
-    MYSQL_DISALLOW();
-  }
-  if (!mysql || tmp) {
-    /* The connection might have been closed */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-  
 #ifdef HAVE_SHUTDOWN_DEFAULT
     /* Mysql 4.1.3 added an extra shutdown_level argument. */
     tmp = mysql_shutdown(mysql, SHUTDOWN_DEFAULT);
@@ -1256,18 +1290,6 @@ static void f_reload(INT32 args)
 
     MYSQL_DISALLOW();
   }
-  if (!mysql || tmp) {
-    /* The connection might have been closed */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-
-    tmp = mysql_reload(mysql);
-
-    MYSQL_DISALLOW();
-  }
 
   if (tmp) {
     Pike_error("Mysql.mysql->reload(): Reload failed\n");
@@ -1290,11 +1312,6 @@ static void f_statistics(INT32 args)
   MYSQL *mysql = PIKE_MYSQL->mysql;
   const char *stats;
 
-  if (!mysql) {
-    pike_mysql_reconnect (1);
-    mysql = PIKE_MYSQL->mysql;
-  }
-
   pop_n_elems(args);
 
   MYSQL_ALLOW();
@@ -1318,11 +1335,6 @@ static void f_server_info(INT32 args)
   MYSQL *mysql = PIKE_MYSQL->mysql;
   const char *info;
 
-  if (!mysql) {
-    pike_mysql_reconnect (1);
-    mysql = PIKE_MYSQL->mysql;
-  }
-
   pop_n_elems(args);
 
   push_text("mysql/");
@@ -1349,10 +1361,6 @@ static void f_host_info(INT32 args)
   MYSQL *mysql;
   const char *info;
 
-  if (!PIKE_MYSQL->mysql) {
-    pike_mysql_reconnect (1);
-  }
-
   mysql = PIKE_MYSQL->mysql;
 
   pop_n_elems(args);
@@ -1381,10 +1389,6 @@ static void f_protocol_info(INT32 args)
   MYSQL *mysql;
   int prot;
 
-  if (!PIKE_MYSQL->mysql) {
-    pike_mysql_reconnect (1);
-  }
-
   pop_n_elems(args);
 
   mysql = PIKE_MYSQL->mysql;
@@ -1436,18 +1440,6 @@ static void f_list_dbs(INT32 args)
 
     MYSQL_DISALLOW();
   }
-  if (!mysql || !result) {
-    /* The connection might have been closed */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-
-    result = mysql_list_dbs(mysql, wild);
-
-    MYSQL_DISALLOW();
-  }
 
   if (!result) {
     const char *err;
@@ -1522,18 +1514,6 @@ static void f_list_tables(INT32 args)
 
     MYSQL_DISALLOW();
   }
-  if (!mysql || !result) {
-    /* The connection might have been closed */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-
-    result = mysql_list_tables(mysql, wild);
-
-    MYSQL_DISALLOW();
-  }
 
   if (!result) {
     const char *err;
@@ -1673,18 +1653,6 @@ static void f_list_fields(INT32 args)
 
     MYSQL_DISALLOW();
   }
-  if (!mysql || !result) {
-    /* The connection might have been closed */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-
-    result = mysql_list_fields(mysql, table, wild);
-
-    MYSQL_DISALLOW();
-  }
 
   if (!result) {
     const char *err;
@@ -1733,18 +1701,6 @@ static void f_list_processes(INT32 args)
 
     MYSQL_DISALLOW();
   }
-  if (!mysql || !result) {
-    /* The connection might have been closed */
-    pike_mysql_reconnect (1);
-
-    mysql = PIKE_MYSQL->mysql;
-
-    MYSQL_ALLOW();
-
-    result = mysql_list_processes(mysql);
-
-    MYSQL_DISALLOW();
-  }
 
   if (!result) {
     const char *err;
@@ -1806,10 +1762,10 @@ static void f_set_charset (INT32 args)
     SIMPLE_ARG_ERROR ("set_charset", 0,
 		      "The charset name cannot contain a NUL character.");
 
+  connection_set_charset (charset);
   if (PIKE_MYSQL->conn_charset)
     free_string (PIKE_MYSQL->conn_charset);
   copy_shared_string (PIKE_MYSQL->conn_charset, charset);
-  connection_set_charset();
   pop_n_elems (args);
 }
 
@@ -1888,6 +1844,10 @@ PIKE_MODULE_INIT
   ADD_FUNCTION("_sprintf",mysql__sprintf,
 	       tFunc(tInt tOr(tVoid,tMapping),tString),0);
   /* function(void:int) */
+  ADD_FUNCTION("is_open", f_is_open, tFunc(tVoid,tInt), ID_PUBLIC);
+  /* function(void:int) */
+  ADD_FUNCTION("ping", f_ping, tFunc(tVoid,tInt), ID_PUBLIC);
+  /* function(void:int) */
   ADD_FUNCTION("affected_rows", f_affected_rows,tFunc(tVoid,tInt), ID_PUBLIC);
   /* function(void:int) */
   ADD_FUNCTION("insert_id", f_insert_id,tFunc(tVoid,tInt), ID_PUBLIC);
