@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 88942e87ffdbce96fea4c90712164b565236bef1 $
+|| $Id: fa2c2c41c85777280fda2ba0af45aa2e6266e6dd $
 */
 
 /*
@@ -765,6 +765,23 @@ static void json_escape(struct string_builder *res,
  	string_builder_putchar(res, '\\');
  	string_builder_putchar(res, 't');
 	break;
+      case 226:
+	if (((i + 2) < len) &&
+	    (str[i+1] == 128) && ((str[i+2] & 0xfe) == 168)) {
+	  /* UTF8-encoded \u2028 or \u2029.
+	   *
+	   * Javascript-based JSON-decoders don't like these
+	   * raw in strings. cf [bug 6103] and others.
+	   */
+	  i += 2;
+	  if (str[i] & 1) {
+	    string_builder_strcat(res, "\\u2029");
+	  } else {
+	    string_builder_strcat(res, "\\u2028");
+	  }
+	  break;
+	}
+	/* FALL_THROUGH */
       default:
 	string_builder_putchar(res, str[i]);
 	break;
