@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: odbc.c,v 1.52 2010/05/11 16:28:02 grubba Exp $
+|| $Id: d5ba28fc688bdf3999e50fb3674c8734b0e8ea01 $
 */
 
 /*
@@ -289,7 +289,8 @@ static void f_create(INT32 args)
 
   check_all_args("odbc->create", args,
 		 BIT_STRING|BIT_INT|BIT_VOID, BIT_STRING|BIT_INT|BIT_VOID,
-		 BIT_STRING|BIT_INT|BIT_VOID, BIT_STRING|BIT_VOID|BIT_INT, 0);
+		 BIT_STRING|BIT_INT|BIT_VOID, BIT_STRING|BIT_VOID|BIT_INT,
+		 BIT_MAPPING|BIT_INT|BIT_VOID, 0);
 
 #define GET_ARG(VAR, ARG) do {					\
     if ((args > (ARG)) && (sp[(ARG)-args].type == T_STRING)) {	\
