@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: postgres.c,v 1.66 2010/02/21 22:42:19 srb Exp $
+|| $Id: 37f66d3210802a497586766f97455aec42362b72 $
 */
 
 /*
@@ -232,18 +232,20 @@ static void f_create (INT32 args)
 {
         /*port will be used as a string with a sprintf()*/
 	char * host=NULL, *db=NULL, *user=NULL, *pass=NULL, *port=NULL;
+	struct mapping *options = NULL;
 	int port_no = -1;
 	char port_buffer[10]; /*it's enough, we need only 6*/
 	PGconn * conn;
 	PQ_FETCH();
 
 	get_all_args("postgres->create",args,
-		     ".%s%s%s%s%d",
+		     ".%s%s%s%s%d%G",
 		     &host,
 		     &db,
 		     &user,
 		     &pass,
-		     &port_no);
+		     &port_no,
+		     &options);
 
 	if (port_no > 0) {
 	  if (port_no < 65536) {
