@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: acconfig.h,v 1.32 2009/07/23 15:15:51 grubba Exp $
+|| $Id: 7f4d1683bb9ddd53ec9a47511398c9e53bc13c0b $
 */
 
 #ifndef FILE_MACHINE_H
@@ -113,4 +113,10 @@
    calling grantpt (needed on SysV's like Solaris) */
 #undef USE_PT_CHMOD
 
+#if (defined(HAVE_STRUCT_MSGHDR_MSG_CONTROL) && defined(SCM_RIGHTS)) ||	\
+    defined(HAVE_STRUCT_MSGHDR_MSG_ACCRIGHTS) || \
+    defined(I_SENDFD)
+#define HAVE_PIKE_SEND_FD
+#endif
+
 #endif
