@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 0743fe4b8692a813357929e59a33116a89dbb2a3 $
+|| $Id: 2530457c6a2549b72022218915a15ddd8ff94479 $
 */
 
 #include "global.h"
@@ -922,9 +922,11 @@ void f_rm(INT32 args)
   {
     if(S_IFDIR == (S_IFMT & st.st_mode))
     {
-      i=rmdir(str->str) != -1;
+      while (!(i = rmdir(str->str) != -1) && (errno == EINTR))
+	;
     }else{
-      i=unlink(str->str) != -1;
+      while (!(i = unlink(str->str) != -1) && (errno == EINTR))
+	;
     }
 #ifdef __NT__
     /* NT looks at the permissions on the file itself and refuses to
@@ -936,9 +938,11 @@ void f_rm(INT32 args)
       else {
 	if(S_IFDIR == (S_IFMT & st.st_mode))
 	{
-	  i=rmdir(str->str) != -1;
+	  while (!(i = rmdir(str->str) != -1) && (errno == EINTR))
+	    ;
 	}else{
-	  i=unlink(str->str) != -1;
+	  while (!(i = unlink(str->str) != -1) && (errno == EINTR))
+	    ;
 	}
 	if (!i) {		/* Failed anyway; try to restore the old mode. */
 	  int olderrno = errno;
@@ -1835,10 +1839,13 @@ void f_mv(INT32 args)
 	  if (!(st.st_mode & _S_IWRITE))
 	    if (chmod (s, st.st_mode | _S_IWRITE))
 	      goto nt_rename_kludge_fail;
-	  if ((st.st_mode & S_IFMT) == S_IFDIR)
-	    err = rmdir (s);
-	  else
-	    err = unlink (s);
+	  if ((st.st_mode & S_IFMT) == S_IFDIR) {
+	    while ((err = rmdir (s)) && (errno == EINTR))
+	      ;
+	  } else {
+	    while ((err = unlink (s)) && (errno == EINTR))
+	      ;
+	  }
 	  if (!err) {		/* Success. */
 	    i = 0;
 	    goto nt_rename_kludge_end;
