@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: file_functions.h,v 1.48 2010/05/29 16:11:27 nilsson Exp $
+|| $Id: c32a9004bd5bcd7850246616cface0a011ab088b $
 */
 
 #define CB_FUNC tFunc(tNone,tOr(tVoid,tMixed))
@@ -34,6 +34,10 @@ FILE_FUNC("write_oob",file_write_oob,
 	       tFuncV(tAttr("sprintf_format", tStr),
 		      tAttr("sprintf_args", tMixed),tInt)))
 
+#ifdef HAVE_PIKE_SEND_FD
+FILE_FUNC("send_fd", file_send_fd, tFunc(tObjIs_STDIO_FD, tInt01))
+#endif
+
 #ifdef HAVE_FSYNC
 /*  function(:int) */
 FILE_FUNC("sync", file_sync, tFunc(tNone,tInt))
@@ -119,7 +123,7 @@ FILE_FUNC("connect_unix",file_connect_unix, tFunc(tStr,tInt))
 /* function(int|void:string) */
 FILE_FUNC("query_address",file_query_address, tFunc(tOr(tInt01,tVoid),tStr))
 /* function(void|string,void|string:void) */
-FILE_FUNC("create",file_create, tFunc(tOr(tVoid,tStr) tOr(tVoid,tStr),tVoid))
+FILE_FUNC("create",file_create, tFunc(tOr3(tVoid,tInt,tStr) tOr(tVoid,tStr) tOr(tVoid,tInt),tVoid))
 /* function(mixed:object) */
 FILE_FUNC("`<<",file_lsh, tAttr("deprecated", tFunc(tMixed,FILE_OBJ)))
 
