@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: socket.c,v 1.112 2010/07/27 16:46:05 mast Exp $
+|| $Id: 60524c48cc65d02e18e1d4404a769d8b1cf629a6 $
 */
 
 #define NO_PIKE_SHORTHAND
@@ -288,6 +288,17 @@ static void port_bind(INT32 args)
   }
 #endif
 
+#if defined(IPV6_V6ONLY) && defined(IPPROTO_IPV6)
+  if (SOCKADDR_FAMILY(addr) == AF_INET6) {
+    /* Attempt to enable dual-stack (ie mapped IPv4 adresses).
+     * Needed on WIN32.
+     * cf http://msdn.microsoft.com/en-us/library/windows/desktop/bb513665(v=vs.85).aspx
+     */
+    int o = 0;
+    fd_setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&o, sizeof(int));
+  }
+#endif
+
   my_set_close_on_exec(fd,1);
 
   THREADS_ALLOW_UID();
