@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: object.c,v 1.303 2009/08/25 16:59:22 grubba Exp $
+|| $Id: 33af7957fb261dc562981b1c2db62683ae481b60 $
 */
 
 #include "global.h"
@@ -114,7 +114,7 @@ PMOD_EXPORT struct object *low_clone(struct program *p)
 
   o=alloc_object();
 
-  o->storage=p->storage_needed ? (char *)xalloc(p->storage_needed) : (char *)NULL;
+  o->storage=p->storage_needed ? (char *)xcalloc(p->storage_needed, 1) : (char *)NULL;
 
   GC_ALLOC(o);
 
@@ -236,7 +236,12 @@ PMOD_EXPORT void call_c_initializers(struct object *o)
   struct program *p=o->prog;
   struct pike_frame *pike_frame=0;
   int frame_pushed = 0;
-  
+
+  /* NOTE: This function is only called for objects straight after
+   *       low_clone(), or after an explicit xcalloc(), which implies
+   *       that the storage (if any) has been zeroed.
+   */
+
   /* clear globals and call C initializers */
   for(e=p->num_inherits-1; e>=0; e--)
   {
@@ -258,20 +263,15 @@ PMOD_EXPORT void call_c_initializers(struct object *o)
 	  struct svalue *s;
 	  s=(struct svalue *)(storage + prog->identifiers[d].func.offset);
 	  s->type=T_INT;
-	  s->u.integer=0;
-	  s->subtype=0;
-	} else {
+#ifdef NEED_CUSTOM_IEEE
+	} else if (prog->identifiers[d].run_time_type == T_FLOAT) {
+	  /* Note: In IEEE representations the value 0.0 is represented as all
+	   *       zeros, and the default initialization is thus sufficient.
+	   */
 	  union anything *u;
 	  u=(union anything *)(storage + prog->identifiers[d].func.offset);
-	  switch(prog->identifiers[d].run_time_type)
-	  {
-	    case PIKE_T_FREE:
-	    case PIKE_T_GET_SET:
-	      break;
-	    case T_INT: u->integer=0; break;
-	    case T_FLOAT: u->float_number=0.0; break;
-	    default: u->refs=0; break;
-	  }
+	  u->float_number=0.0;
+#endif /* NEED_CUSTOM_IEEE */
 	}
 	(void) debug_malloc_update_location(o, DMALLOC_NAMED_LOCATION(" clear_global"));
       }
@@ -658,6 +658,7 @@ PMOD_EXPORT struct object *get_master(void)
   }
 
   {
+    int f;
     ONERROR uwp;
 
     /* fprintf(stderr, "Cloning master...\n"); */
@@ -673,7 +674,11 @@ PMOD_EXPORT struct object *get_master(void)
 
     call_c_initializers(master_object);
     call_pike_initializers(master_object,0);
-  
+
+    f = find_identifier ("is_pike_master", master_program);
+    if (f >= 0)
+      object_low_set_index (master_object, f, &svalue_int_one);
+
     /* fprintf(stderr, "Master loaded.\n"); */
 
     UNSET_ONERROR (uwp);
@@ -807,7 +812,11 @@ PMOD_EXPORT void destruct_object (struct object *o, enum object_destruct_reason
   SET_ONERROR(uwp, fatal_on_error,
 	      "Shouldn't get an exception in destruct().\n");
   if(d_flag > 20) do_debug();
+
+  if (Pike_in_gc >= GC_PASS_PRETOUCH && Pike_in_gc < GC_PASS_FREE)
+    gc_fatal (o, 1, "Destructing objects is not allowed inside the gc.\n");
 #endif
+
 #ifdef GC_VERBOSE
   if (Pike_in_gc > GC_PASS_PREPARE) {
     fprintf(stderr, "|   Destructing %p with %d refs", o, o->refs);
@@ -957,7 +966,10 @@ void low_destruct_objects_to_destruct(void)
    * to avoid that reentrant calls to this function go through all
    * objects instead of just the newly added ones. This way we avoid
    * extensive recursion in this function and also avoid destructing
-   * the objects arbitrarily late. */
+   * the objects arbitrarily late.
+   *
+   * Note that we might start a gc during this, but since the unlinked
+   * objects have no refs, the gc never encounters them. */
   while (objects_to_destruct) {
     o = objects_to_destruct, objects_to_destruct = 0;
 
@@ -1020,8 +1032,10 @@ PMOD_EXPORT void schedule_really_free_object(struct object *o)
 #endif
     Pike_fatal("Object got %d references in schedule_really_free_object().\n", o->refs);
   }
-  if (Pike_in_gc > GC_PASS_PREPARE && Pike_in_gc < GC_PASS_FREE && o->next != o)
-    Pike_fatal("Freeing objects is not allowed inside the gc.\n");
+  if (Pike_in_gc > GC_PASS_PREPARE && Pike_in_gc < GC_PASS_FREE &&
+      /* Fake objects are invisible to the gc. */
+      o->next != o)
+    gc_fatal(o, 0, "Freeing objects is not allowed inside the gc.\n");
 #endif
 
   debug_malloc_touch(o);
@@ -1045,7 +1059,9 @@ PMOD_EXPORT void schedule_really_free_object(struct object *o)
   if(o->prog)
   {
     o->next = objects_to_destruct;
-    DO_IF_DMALLOC(o->prev = (void *) -1);
+#ifdef PIKE_DEBUG
+    o->prev = (void *) -3;
+#endif
     PIKE_MEM_WO(o->prev);
     objects_to_destruct = o;
 
@@ -1105,6 +1121,9 @@ PMOD_EXPORT void schedule_really_free_object(struct object *o)
  * NOTE: This function may be called by the compiler on objects
  *       cloned from unfinished programs (ie placeholder
  *       objects). Degenerated cases may thus occur.
+ *       It may also be called via lfuns in the currently
+ *       compiling program (notably lfun::`->()) and thus
+ *       execute in place-holder objacts.
  */
 PMOD_EXPORT void low_object_index_no_free(struct svalue *to,
 					  struct object *o,
@@ -1207,7 +1226,9 @@ PMOD_EXPORT void low_object_index_no_free(struct svalue *to,
       } else {
 	Pike_error("No getter for variable %S.\n", i->name);
       }
-    } else if ((i->run_time_type == PIKE_T_FREE) || !PIKE_OBJ_STORAGE(o)) {
+    } else if (!(p->flags & PROGRAM_FINISHED) ||
+	       (i->run_time_type == PIKE_T_FREE) ||
+	       !PIKE_OBJ_STORAGE(o)) {
       /* Variable storage not allocated. */
 #ifdef PIKE_DEBUG
       if ((i->run_time_type != PIKE_T_FREE) && (p->flags & PROGRAM_FINISHED)) {
@@ -1217,7 +1238,7 @@ PMOD_EXPORT void low_object_index_no_free(struct svalue *to,
       to->type = T_INT;
       to->subtype = ((i->run_time_type == PIKE_T_FREE)?
 		     NUMBER_UNDEFINED:NUMBER_NUMBER);
-      to->u.integer = 0;      
+      to->u.integer = 0;
     } else {
       void *ptr=LOW_GET_GLOBAL(o,f,i);
       switch(i->run_time_type)
@@ -2004,7 +2025,7 @@ PMOD_EXPORT void visit_object (struct object *o, int action)
   }
 }
 
-PMOD_EXPORT void visit_function (struct svalue *s, int ref_type)
+PMOD_EXPORT void visit_function (const struct svalue *s, int ref_type)
 {
 #ifdef PIKE_DEBUG
   if (s->type != T_FUNCTION)
@@ -2027,7 +2048,7 @@ PMOD_EXPORT void gc_mark_object_as_referenced(struct object *o)
   debug_malloc_touch(o);
   debug_malloc_touch(o->storage);
 
-  if(gc_mark(o)) {
+  if(gc_mark(o, T_OBJECT)) {
     if(o->next == o) return; /* Fake object used by compiler */
 
     GC_ENTER (o, T_OBJECT) {
@@ -2595,7 +2616,7 @@ static void f_magic_indices (INT32 args)
 	prog = obj->prog;
       }
       pop_n_elems (args);
-      push_array (res = allocate_array_no_init (prog->num_identifier_references, 0));
+      push_array (res = allocate_array(prog->num_identifier_references));
       for (e = i = 0; e < (int) prog->num_identifier_references; e++) {
 	struct reference *ref = prog->identifier_references + e;
 	struct identifier *id = ID_FROM_PTR (prog, ref);
@@ -2605,8 +2626,9 @@ static void f_magic_indices (INT32 args)
 	copy_shared_string (ITEM(res)[i].u.string, ID_FROM_PTR (prog, ref)->name);
 	ITEM(res)[i++].type = T_STRING;
       }
-      res->type_field = BIT_STRING;
+      res->type_field |= BIT_STRING;
       sp[-1].u.array = resize_array (res, i);
+      res->type_field = BIT_STRING;
       return;
     case 2:
       prog = obj->prog;
@@ -2670,9 +2692,9 @@ static void f_magic_values (INT32 args)
 	inherit = prog->inherits + 0;
       }
       pop_n_elems (args);
-      push_array (res = allocate_array_no_init (prog->num_identifier_references, 0));
+      push_array (res = allocate_array(prog->num_identifier_references));
       types = 0;
-      for (e = i = 0; e < (int) prog->num_identifier_references; e++, i++) {
+      for (e = i = 0; e < (int) prog->num_identifier_references; e++) {
 	struct reference *ref = prog->identifier_references + e;
 	struct identifier *id = ID_FROM_PTR (prog, ref);
 	if (ref->id_flags & ID_HIDDEN) continue;
@@ -2680,10 +2702,11 @@ static void f_magic_values (INT32 args)
 	    (ID_INHERITED|ID_PRIVATE)) continue;
 	low_object_index_no_free (ITEM(res) + i, obj,
 				  e + inherit->identifier_level);
-	types |= 1 << ITEM(res)[i].type;
+	types |= 1 << ITEM(res)[i++].type;
       }
-      res->type_field = types;
+      res->type_field |= types;
       sp[-1].u.array = resize_array (res, i);
+      res->type_field = types;
       return;
     case 2:
       prog = obj->prog;
