@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: pike_threadlib.h,v 1.73 2010/09/28 16:13:32 grubba Exp $
+|| $Id: 8ec30b55f7630bf5303666ccdcca8350b6787a01 $
 */
 
 #ifndef PIKE_THREADLIB_H
@@ -88,6 +88,9 @@ PMOD_EXPORT extern struct program *thread_id_prog;
 #include <sched.h>
 #endif
 
+#ifdef USE_DARWIN_THREADS_WITHOUT_MACH
+/* OSX Threads don't get along with mach headers! */
+#else
 #ifdef HAVE_MACH_TASK_INFO_H
 #include <mach/task_info.h>
 #endif
@@ -97,6 +100,7 @@ PMOD_EXPORT extern struct program *thread_id_prog;
 #ifdef HAVE_MACH_MACH_INIT_H
 #include <mach/mach_init.h>
 #endif
+#endif /* USE_DARWIN_THREADS_WITHOUT_MACH */
 
 
 /* Restore the fp macro. */
