@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: db5a9618be229a5c4cd1539c506150dfdd26ad7d $
+|| $Id: c01896599e3327a625487d529aa5210474b39539 $
 */
 
 #include "global.h"
@@ -1298,7 +1298,7 @@ INPUT_IS_WIDE(								 \
       check_stack(1);							 \
       *sp++=sval;							 \
       dmalloc_touch_svalue(Pike_sp-1);					 \
-      DO_IF_DEBUG(sval.type=99);					 \
+      DO_IF_DEBUG(INVALIDATE_SVAL(sval));				 \
     }									 \
   }									 \
   chars_matched[0]=eye;							 \
