@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: stralloc.c,v 1.239 2010/06/15 16:10:19 grubba Exp $
+|| $Id: b9bba00755e7e288a4e0ac720bccc3a9d9b47134 $
 */
 
 #include "global.h"
@@ -2146,6 +2146,18 @@ static INLINE size_t memory_in_string (struct pike_string *s)
     return sizeof (struct pike_string_hdr) + ((s->len + 1) << s->size_shift);
 }
 
+void count_memory_in_short_pike_strings(size_t *num, size_t *size)
+{
+  size_t num_=0, size_=0;
+  count_memory_in_short_pike_string0s(num, size);
+  count_memory_in_short_pike_string1s(&num_, &size_);
+  *num += num_;
+  *size += size_;
+  count_memory_in_short_pike_string2s(&num_, &size_);
+  *num += num_;
+  *size += size_;
+}
+
 void count_memory_in_strings(size_t *num, size_t *size)
 {
   unsigned INT32 e;
