@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: de735eba22170eb7e2bd99b14346ebe5190a754c $
+|| $Id: 4629bd7003bea8151c0925b0c7ec6c29af7428ed $
 */
 
 #ifndef SVALUE_H
@@ -100,6 +100,19 @@ struct svalue
   union anything u; /**< contains the value */
 };
 
+#define TYPEOF(SVAL)	((SVAL).type)
+#define SUBTYPEOF(SVAL)	((SVAL).subtype)
+#define SET_SVAL_TYPE(SVAL, TYPE)	(TYPEOF(SVAL) = (TYPE))
+#define SET_SVAL_SUBTYPE(SVAL, TYPE)	(SUBTYPEOF(SVAL) = (TYPE))
+#define SET_SVAL(SVAL, TYPE, SUBTYPE, FIELD, EXPR) do {	\
+    /* Set the type afterwards to avoid a clobbered	\
+     * svalue in case EXPR throws. */			\
+    (SVAL).u.FIELD = (EXPR);				\
+    SET_SVAL_TYPE((SVAL), (TYPE));			\
+    SET_SVAL_SUBTYPE((SVAL), (SUBTYPE));		\
+  } while(0)
+#define INVALIDATE_SVAL(SVAL) SET_SVAL_TYPE(SVAL, 99) /* an invalid type */
+
 #define PIKE_T_ARRAY 0
 #define PIKE_T_MAPPING 1
 #define PIKE_T_MULTISET 2
@@ -701,9 +714,7 @@ static INLINE struct callable *pass_callable (struct callable *c) {return c;}
   ptrdiff_t num_=(Y);				\
   for(;num_-- > 0;s_++)				\
   {						\
-      s_->type=PIKE_T_INT;			\
-      s_->subtype=(N);				\
-      s_->u.integer=0;				\
+    SET_SVAL(*s_, PIKE_T_INT, (N), integer, 0);	\
   }						\
 }while(0)
 
@@ -882,8 +893,7 @@ static INLINE void free_svalue(struct svalue *s)
 {
   INT64 tmp;
   struct svalue zero;
-  zero.type=PIKE_T_INT;
-  zero.subtype = NUMBER_NUMBER;
+  SET_SVAL(zero, PIKE_T_INT, NUMBER_NUMBER, integer, 0);
   tmp=pike_atomic_swap64((INT64 *)s, *(INT64 *)&zero);
   free_svalue_unlocked((struct svalue *)&tmp);
 }
